#!/bin/bash

XML="1.2.xml"
DTD="1.2.dtd"
XSD="1.2.xsd"
XSD1="1.1.xsd"

function die {
  echo "ERROR: $@"
  exit 1
}

function check_file {
  [[ -f "$1" ]] || die "FILE '$1' not found"
}

function strip {
  s=$(cat)
  s=${s## }
  echo ${s%% }
}

check_file "$XML"

xmllint --version 2> /dev/null || die "xmllint not found. Please install it."
xsltproc --version > /dev/null || die "xsltproc not found. Please install it."

xmllint --noout "$XML" || die "File '$XML' is not a valid XML file."
echo "XML Test 1 PASSED: '$XML' is a valid XML file."

check_file "$XSD"
xmllint --noout --schema "$XSD" "$XML" \
  || die "File '$XML' was not valid for schema '$XSD'"
echo "XML Test 2 PASSED: '$XML' is valid for schema '$XSD'"

check_file "$XSD1"
xmllint --noout --schema "$XSD1" "$XML" 2> /dev/null \
  && die "File '$XML' was valid for schema '$XSD1'"
echo "XML Test 3 PASSED: '$XML' is invalid for schema '$XSD1'"
echo "XML Tests: SUCCESS"
echo "------"

check_file "$DTD"
xmllint --noout --dtdvalid "$DTD" "$XML"\
  || die "Your file '$XML' was not valid with your DTD '$DTD'."
echo "DTD Test 1 PASSED: '$XML' is valid for DTD '$DTD'"

VALID="1.2.valid*.xml"
NUM=$(ls $VALID | wc -l | strip)
(( NUM >= 2 )) || die "Missing test files? Found only $NUM files matching $VALID"
for i in $VALID; do
  xmllint --noout --schema "$XSD" "$i" 2> /dev/null \
    || die "Bug in the auto-test with file '$i'"
  xmllint --noout --dtdvalid "$DTD" "$i"\
    || die "ERROR: '$i' is valid for the reference schema '$XSD', but not for your DTD '$DTD'"
done
INVALID="1.2.invalid*.xml"
NUM=$(ls $INVALID | wc -l | strip)
(( NUM >= 2 )) || die "Missing test files? Found only $NUM files matching $INVALID"
for i in $INVALID; do
  xmllint --noout --schema "$XSD" "$i" 2> /dev/null \
    && die "Bug in the auto-test with file '$i'"
  xmllint --noout --dtdvalid "$DTD" "$i" 2> /dev/null \
    && die "ERROR: '$i' is invalid for the reference schema '$XSD', but is valid for your DTD '$DTD'"
done
echo "DTD Test 2 PASSED: your DTD seems as 'equivalent' to the reference schema as it can be."
echo "DTD Tests: SUCCESS"
